/* Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.sinfo.collision;

import java.util.ArrayList;
import java.util.Collection;

import org.apache.commons.collections.Predicate;
import org.dive4elements.river.artifacts.common.AbstractCalculationExportableResult;
import org.dive4elements.river.artifacts.common.ExportContextCSV;
import org.dive4elements.river.artifacts.common.ExportContextPDF;
import org.dive4elements.river.artifacts.common.GeneralResultType;
import org.dive4elements.river.artifacts.common.IResultType;
import org.dive4elements.river.artifacts.common.MetaAndTableJRDataSource;
import org.dive4elements.river.artifacts.common.ResultRow;
import org.dive4elements.river.artifacts.sinfo.common.SInfoResultType;
import org.dive4elements.river.artifacts.sinfo.util.RiverInfo;

/**
 * Contains the results of a {@link CollisionCalculation}.
 *
 * @author Gernot Belger
 */
final public class CollisionCalcOverviewResult extends AbstractCalculationExportableResult {

    private static final long serialVersionUID = 1L;

    private static final String JASPER_FILE = "/jasper/templates/sinfo.collision.overview.jrxml";

    private final int[] singleYears;

    public CollisionCalcOverviewResult(final String label, final int[] singleYears, final Collection<ResultRow> rows) {
        super(label, rows);
        this.singleYears = singleYears;
    }

    public int[] getSingleYears() {
        return this.singleYears;
    }

    public double[][] getStationPointsByYear(final IResultType type, final int year) {

        final Predicate filter = new Predicate() {
            @Override
            public boolean evaluate(final Object object) {
                final ResultRow row = (ResultRow) object;
                return getSingleYears() == null || Integer.valueOf(row.getValue(SInfoResultType.years).toString()) == year;
            }
        };

        return getPoints(GeneralResultType.station, type, filter);
    }

    @Override
    protected void writeCSVResultMetadata(final ExportContextCSV exportContextCSV) {
        // do nothing so far
    }

    @Override
    public void writeCSVHeader(final ExportContextCSV exportContextCSV, final RiverInfo river) {
        final int colSize = 3;
        exportContextCSV.writeTitleForTabs("sinfo.export.csv.title.collison.overview", 3); // Voraussetzung für Tabs ist, dass der Titel vor den Headern
        // geschrieben wird.
        // Das ist etwas doof.

        final Collection<String> header = new ArrayList<>(colSize);

        header.add(exportContextCSV.formatCsvHeader(GeneralResultType.station));
        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.years));
        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.collisionCount));

        exportContextCSV.writeCSVLine(header.toArray(new String[colSize]));
    }

    @Override
    protected String[] formatCSVRow(final ExportContextCSV exportContextCSV, final ResultRow row) {
        return formatRow(exportContextCSV, row);
    }

    @Override
    protected String[] formatPDFRow(final ExportContextPDF exportContextPDF, final ResultRow row) {

        final Collection<String> lines = new ArrayList<>(3);

        lines.add(exportContextPDF.formatRowValue(row, GeneralResultType.station));
        lines.add(exportContextPDF.formatRowValue(row, SInfoResultType.years));
        lines.add(exportContextPDF.formatRowValue(row, SInfoResultType.collisionCount));

        return lines.toArray(new String[lines.size()]);
    }

    private String[] formatRow(final ExportContextCSV exportContextCSV, final ResultRow row) {

        final Collection<String> lines = new ArrayList<>(3);

        lines.add(exportContextCSV.formatRowValue(row, GeneralResultType.station));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.years));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.collisionCount));

        return lines.toArray(new String[lines.size()]);
    }

    @Override
    protected String getJasperFile() {
        return JASPER_FILE;
    }

    @Override
    protected void addJRTableHeader(final ExportContextPDF exportContext, final MetaAndTableJRDataSource source) {

        /* column headings */
        exportContext.addJRMetadata(source, "station_header", GeneralResultType.station);
        exportContext.addJRMetadata(source, "collision_years_header", SInfoResultType.years);
        exportContext.addJRMetadata(source, "collision_count_header", SInfoResultType.collisionCount);
    }
}